//	astrosource.c: Implementation of BRELS AstroSource
//	Breno Sarmento
//	breno_sarmento@hotmail.com
//	http://www27.brinkster.com/brels/
//	Started: March 24th, 2003
//	Version: May 30th, 2003
//	Version: November 24th, 2003

#define USE_DLL

#include <windows.h>
#include <swephexp.h>

const double PI = 3.1415926535897932384626433832795;

#define IDNOW 10001
#define IDABOUT 10002

WNDCLASS wc;
HWND window;
HICON BRELS;
MSG msg;
HWND edta, edtm, edtd, edth, edtmin, edtseg, edtgmt,edtlatg, edtlatm, 
     edtlong, edtlonm, memo, map, btcalc, btnow, btabout;
HDC hdc, mapdc;

int year, month, day, hour, minute, second, gmt; 
int i, j, k, temp;
char temp1[4096]; char* lptemp1 = (char*) &temp1;
char temp2[4096]; char* lptemp2 = (char*) &temp2;
char temp3[4096]; char* lptemp3 = (char*) &temp3;

int fire, water, earth, air, cardinal, fixed, mutable_, positive, negative;
int north, south, east, west, angular, succedant, cadent;


int elements[4] = {0xFF0000, 0x0000FF, 0x00FFFF, 0x00FF00}; // Wat Fir Ear Air

char* Signs[12]={
			"Ari", "Tau", "Gem", "Can", "Leo", "Vir", 
			"Lib", "Sco", "Sag", "Cap", "Aqu", "Pis"
		     };

char* planet_name[14];

int aspects[12];
double move[11], planets[13], positions[6], base;
double ascmc[10], cusps[13], jday, jhour, lat, latmin, lon, lonmin;

LOGBRUSH lb;

double Convert(double degree)
{
	if ((degree+base)<360)
		return -(degree+base)*PI/180.0;
	else
		return -(degree+base-360)*PI/180.0;
};

double Signal(double value)
{
	if (value<0) return -1.0; else return 1.0;
};

double Sign(double degree, int option)
{
	switch (option)
	{
		case 0:
			return (floor(degree/30.0)+ 1);
		case 1:
			return floor(degree-floor(degree/30.0)*30);
		case 2:
			return (degree-floor(degree))*60.0;
		default:
			return -1;
	};
};

int House(double degree)
{
	double degree1, degree2, degree3;
	int i1;
	for (i1=1; i1<=12; i1++)
	{
		degree3=degree;
		degree1=cusps[i1];
		if (i1==12) degree2=cusps[1]; else degree2=cusps[i1+1];
		if ((degree2< degree1) && (degree2< degree3)) degree2=degree2+360;
		if ((degree1> degree2) && (degree1> degree3)) degree1=degree1-360;
		if ((degree3>=degree1) && (degree3<=degree2)) return i1;
	};
	return -1;
};

void Calculate(void)
{
	
	DeleteObject(SelectObject(mapdc, CreateFont(24,0,0,0,FW_NORMAL,FALSE,FALSE,FALSE,DEFAULT_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,ANTIALIASED_QUALITY,DEFAULT_PITCH,"Astro")));
	SetBkMode(mapdc, TRANSPARENT);
	SetTextAlign(mapdc, TA_CENTER);

	ZeroMemory(&move, sizeof(move));
	ZeroMemory(&planets, sizeof(planets));
	ZeroMemory(&aspects, sizeof(aspects));
	planet_name[SE_SUN]="Sun";
	planet_name[SE_MOON]="Moon";
	planet_name[SE_MERCURY]="Mercury";
	planet_name[SE_VENUS]="Venus";
	planet_name[SE_MARS]="Mars";
	planet_name[SE_JUPITER]="Jupiter";
	planet_name[SE_SATURN]="Saturn";
	planet_name[SE_URANUS]="Uranus";
	planet_name[SE_NEPTUNE]="Neptune";
	planet_name[SE_PLUTO]="Pluto";
	planet_name[SE_MEAN_NODE]="Node";
	planet_name[SE_MEAN_NODE+1]="ASC";
	planet_name[SE_MEAN_NODE+2]="M.C.";
	fire=0; water=0; earth=0; air=0; 
	cardinal=0; fixed=0; mutable_=0;
	positive=0; negative=0;
	north=0; south=0; east=0; west=0;
	angular=0; succedant=0; cadent=0;

	// Julian date
	GetWindowText(edta, lptemp1,256); year = atoi(lptemp1);
	GetWindowText(edtm, lptemp1,256); month = atoi(lptemp1);
	GetWindowText(edtd, lptemp1,256); day = atoi(lptemp1);
	GetWindowText(edth, lptemp1,256); hour = atoi(lptemp1);
	GetWindowText(edtmin, lptemp1,256); minute = atoi(lptemp1);
	GetWindowText(edtseg, lptemp1,256); second = atoi(lptemp1);
	GetWindowText(edtgmt, lptemp1,256); gmt = atoi(lptemp1);
	jhour = (hour*3600.0 + minute*60.0 + second - gmt * 3600.0) / 3600.0;
	jday = swe_julday(year, month, day, jhour, SE_GREG_CAL);

	// Cusps of the houses
	GetWindowText(edtlatg, lptemp1, 256); 
	lat = atoi(lptemp1);
	GetWindowText(edtlatm, lptemp1, 256); 
	latmin = atoi(lptemp1);
	GetWindowText(edtlong, lptemp1, 256); 
	lon = atoi(lptemp1);
	GetWindowText(edtlonm, lptemp1, 256); 
	lonmin = atoi(lptemp1);
	lat = (fabs(lat) + latmin / 60.0)*Signal(lat);
	lon = (fabs(lon) + lonmin / 60.0)*Signal(lon);
	swe_houses(jday, lat, lon, 'P', (double*) &cusps, (double*) &ascmc);
	base = 180-ascmc[SE_ASC];
	planets[SE_MEAN_NODE+1]=ascmc[SE_ASC]; 
	planets[SE_MEAN_NODE+2]=ascmc[SE_MC]; 

	// Draws the wheel
	DeleteObject(SelectObject(mapdc, CreatePen(PS_NULL,1,0xFFFFFF)));
	Rectangle(mapdc,0,0,500,500);
	lb.lbColor = 0x000000;
	lb.lbStyle = BS_SOLID;
	DeleteObject(SelectObject(mapdc, CreateBrushIndirect(&lb)));
	DeleteObject(SelectObject(mapdc, CreatePen(PS_SOLID,1,0xFFFFFF)));
	Ellipse(mapdc,0,0,499,499);
	for (i=0; i<=11; i++)
	{
		temp1[0]=65+i; temp1[1]=0;
		SetTextColor(mapdc, elements[(i+1) % 4]);
		TextOut(mapdc, cos(Convert(i*30+15))*233+250,sin(Convert(i*30+15))*235+240,lptemp1, 1);
		MoveToEx(mapdc, 250,250, NULL);
		LineTo(mapdc, cos(Convert(i*30))*250+250, sin(Convert(i*30))*250+250);
	};
	lb.lbColor = 0xFFFFFF;
	DeleteObject(SelectObject(mapdc, CreateBrushIndirect(&lb)));
	DeleteObject(SelectObject(mapdc, CreatePen(PS_SOLID,1,0x000000)));
	SetTextColor(mapdc, 0x000000);
	Ellipse(mapdc,30,30,470,470);

	// Draws the houses
	Ellipse(mapdc, 130,130,370,370);
	DeleteObject(SelectObject(mapdc, CreatePen(PS_SOLID,1,0x808080)));
	DeleteObject(SelectObject(mapdc, CreateFont(20,0,0,0,FW_NORMAL,FALSE,FALSE,FALSE,DEFAULT_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,ANTIALIASED_QUALITY,DEFAULT_PITCH,"Astro")));
	for (i=1; i<13; i++)
	{
		if ((i==10) || (i==1))
		{
			DeleteObject(SelectObject(mapdc, CreatePen(PS_SOLID,2,0x000000)));
		}
		else
			DeleteObject(SelectObject(mapdc, CreatePen(PS_SOLID,1,0x808080)));
		TextOut(mapdc, cos(Convert(cusps[i]+15))*110+250,sin(Convert(cusps[i]+15))*110+240, itoa(i, lptemp1,10), i / 10 +1);
		MoveToEx(mapdc, 250,250,NULL);
		LineTo(mapdc, cos(Convert(cusps[i]))*219+250, sin(Convert(cusps[i]))*219+250);
	};
	Ellipse(mapdc, 150,150,350,350);
	DeleteObject(SelectObject(mapdc, CreatePen(PS_SOLID,1,0x000000)));

	// Gets the planets
	for (i=SE_SUN; i<=SE_MEAN_NODE; i++)
	{
		swe_calc_ut(jday, i, 0, (double*) &positions, lptemp1);
		planets[i]=positions[0];
	};

	// Adjusts the planets
	DeleteObject(SelectObject(mapdc, CreateFont(24,0,0,0,FW_NORMAL,FALSE,FALSE,FALSE,DEFAULT_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,ANTIALIASED_QUALITY,DEFAULT_PITCH,"Astro")));
	for (i=SE_SUN; i<SE_MEAN_NODE; i++)
	{
		for (j=i+1; j<=SE_MEAN_NODE; j++)
		{
			k=abs(planets[i]-planets[j]);
			if ((k<=9) || (k>=351)) move[j]+=20;
		};
	};

	// Draws the planets
	for (i=SE_SUN; i<=SE_MEAN_NODE; i++)
	{
		temp1[0]='Q' + i; 
		if (temp1[0]=='[') temp1[0]='<'; 
		temp1[1]=0;

		// Pushes the planet picture for a better viewing
		SetTextColor(mapdc, elements[(int) Sign(planets[i],0) % 4] & 0xFF80FF);
		TextOut(mapdc, cos(Convert(planets[i]))*(200-move[i])+250,sin(Convert(planets[i]))*(200-move[i])+240,lptemp1,1);
	};

	// Calculates rythms and qualities
	temp1[0]=0;
	temp2[0]=0;
	for (i=SE_SUN; i<=SE_MEAN_NODE+1; i++)
	{
		if (i<=SE_MEAN_NODE) switch ((int) House(planets[i]))
		{
			case  1: angular++  ; north++; east++; break;
			case  2: succedant++; north++; east++; break;
			case  3: cadent++   ; north++; east++; break;
			case  4: angular++  ; north++; west++; break;
			case  5: succedant++; north++; west++; break;
			case  6: cadent++   ; north++; west++; break;
			case  7: angular++  ; south++; west++; break;
			case  8: succedant++; south++; west++; break;
			case  9: cadent++   ; south++; west++; break;
			case 10: angular++  ; south++; east++; break;
			case 11: succedant++; south++; east++; break;
			case 12: cadent++   ; south++; east++; break;
		};
		switch((int) Sign(planets[i],0))
		{
			case  1: fire++ ; cardinal++; positive++; break;
			case  2: earth++; fixed++   ; negative++; break;
			case  3: air++  ; mutable_++; positive++; break;
			case  4: water++; cardinal++; negative++; break;
			case  5: fire++ ; fixed++   ; positive++; break;
			case  6: earth++; mutable_++; negative++; break;
			case  7: air++  ; cardinal++; positive++; break;
			case  8: water++; fixed++   ; negative++; break;
			case  9: fire++ ; mutable_++; positive++; break;
			case 10: earth++; cardinal++; negative++; break;
			case 11: air++  ; fixed++   ; positive++; break;
			case 12: water++; mutable_++; negative++; break;
		};
	};

	strcat(lptemp1, "Chart details:\r\n");
	strcat(lptemp1, "\r\nFire:\t"    ); strcat(lptemp1, itoa(fire , lptemp2,10));
	strcat(lptemp1, "\r\nEarth:\t"   ); strcat(lptemp1, itoa(earth, lptemp2,10));
	strcat(lptemp1, "\r\nAir:\t"     ); strcat(lptemp1, itoa(air  , lptemp2,10));
	strcat(lptemp1, "\r\nWater:\t"   ); strcat(lptemp1, itoa(water, lptemp2,10));
	strcat(lptemp1, "\r\n");
	strcat(lptemp1, "\r\nCardinal:\t"); strcat(lptemp1, itoa(cardinal, lptemp2,10));
	strcat(lptemp1, "\r\nFixed:\t"   ); strcat(lptemp1, itoa(fixed   , lptemp2,10));
	strcat(lptemp1, "\r\nMutable:\t" ); strcat(lptemp1, itoa(mutable_, lptemp2,10));
	strcat(lptemp1, "\r\nPositive:\t"); strcat(lptemp1, itoa(positive, lptemp2,10));
	strcat(lptemp1, "\r\nNegative:\t"); strcat(lptemp1, itoa(negative, lptemp2,10));
	strcat(lptemp1, "\r\n");
	strcat(lptemp1, "\r\nAngular:\t" ); strcat(lptemp1, itoa(angular  , lptemp2,10));
	strcat(lptemp1, "\r\nSucc.:\t"   ); strcat(lptemp1, itoa(succedant, lptemp2,10));
	strcat(lptemp1, "\r\nCadent:\t"  ); strcat(lptemp1, itoa(cadent   , lptemp2,10));
	strcat(lptemp1, "\r\n");
	strcat(lptemp1, "\r\nNorth:\t"   ); strcat(lptemp1, itoa(north, lptemp2,10));
	strcat(lptemp1, "\r\nWest:\t"    ); strcat(lptemp1, itoa(west , lptemp2,10));
	strcat(lptemp1, "\r\nEast:\t"    ); strcat(lptemp1, itoa(east , lptemp2,10));
	strcat(lptemp1, "\r\nSouth:\t"   ); strcat(lptemp1, itoa(south, lptemp2,10));
	strcat(lptemp1, "\r\n\r\n");

	// Planets in the signs
	for (i=SE_SUN; i<=SE_MEAN_NODE+2; i++)
	{
		strcat(lptemp1, planet_name[i]);
		strcat(lptemp1, ":\t");
		if (Sign(planets[i], 1)<10) strcat(lptemp1, "0");
		strcat(lptemp1, itoa((int) Sign(planets[i], 1), lptemp2, 10));
		strcat(lptemp1, " ");
		strcat(lptemp1, Signs[(int) Sign(planets[i], 0)-1]);
		strcat(lptemp1, " ");
		if (Sign(planets[i], 2)<10) strcat(lptemp1, "0");
		strcat(lptemp1, itoa((int) Sign(planets[i], 2), lptemp2, 10));
		strcat(lptemp1, "' ");
		if (i<=SE_MEAN_NODE)
		{
			strcat(lptemp1, "\t(");
			strcat(lptemp1, itoa(House(planets[i]), lptemp2,10));
			strcat(lptemp1, ")");
		};
		strcat(lptemp1, "\r\n");
	};

	// Houses in the signs
	strcat(lptemp1, "\r\n");
	for (i=1; i<=12; i++)
	{
		strcat(lptemp1, "House ");
		if (i<10) strcat(lptemp1, "0");
		strcat(lptemp1, itoa(i, lptemp2, 10));
		strcat(lptemp1, ": ");
		if (Sign(cusps[i],1)<10) strcat(lptemp1, "0");
		strcat(lptemp1, itoa((int) Sign(cusps[i],1), lptemp2, 10));
		strcat(lptemp1, " ");
		strcat(lptemp1, Signs[(int) Sign(cusps[i],0)-1]);
		strcat(lptemp1, " ");
		if (Sign(cusps[i],2)<10) strcat(lptemp1, "0");
		strcat(lptemp1, itoa((int) Sign(cusps[i],2),lptemp2,10) );
		strcat(lptemp1, "\r\n");
	};

	// Lists and draws aspects
	strcat(lptemp1, "\r\nAspects\r\n\r\n");
	for (i=SE_SUN; i<=SE_MEAN_NODE+1; i++)
	for (j=i+1; j<=SE_MEAN_NODE+2; j++)
	{
		k=abs(planets[i]-planets[j]);
		temp=0;
		// Conjunction 0 - 360
		if ((k>=0 && k<=7) || (k>=353 && k<=360))
		{
			strcat(lptemp1, planet_name[i]);
			strcat(lptemp1, " Conjunct ");
			strcat(lptemp1, planet_name[j]);
			strcat(lptemp1, "\r\n");
			aspects[i]++;
			aspects[j]++;
		};
		// Trine 120 - 240
		if ((k>=113 && k<=127) || (k>=233 && k<=247))
		{
			strcat(lptemp1, planet_name[i]);
			strcat(lptemp1, " Trine ");
			strcat(lptemp1, planet_name[j]);
			strcat(lptemp1, "\r\n");
			temp=1;
			if ((k==120) || (k==240)) temp=2;
			DeleteObject(SelectObject(mapdc, CreatePen(PS_SOLID,temp,0x00FF00)));
			aspects[i]++;
			aspects[j]++;
		};
		// Square 90 - 270
		if ((k>=83 && k<=97) || (k>=263 && k<=277))
		{
			strcat(lptemp1, planet_name[i]);
			strcat(lptemp1, " Square ");
			strcat(lptemp1, planet_name[j]);
			strcat(lptemp1, "\r\n");
			temp=1;
			if ((k==90) || (k==270)) temp=2;
			DeleteObject(SelectObject(mapdc, CreatePen(PS_SOLID,temp,0x0000FF)));
			aspects[i]++;
			aspects[j]++;
		};
		// Sextile 60 - 300
		if ((k>=56 && k<=64) || (k>=296 && k<=304))
		{
			strcat(lptemp1, planet_name[i]);
			strcat(lptemp1, " Sextile ");
			strcat(lptemp1, planet_name[j]);
			strcat(lptemp1, "\r\n");
			temp=1;
			if ((k==60) || (k==300)) temp=2;
			DeleteObject(SelectObject(mapdc, CreatePen(PS_SOLID,temp,0xFF8000)));
			aspects[i]++;
			aspects[j]++;
		};
		// Semi-sextile 30 - 330
		if (((k>=29) && (k<=30)) || ((k>=329) && (k<=330)))
		{
			strcat(lptemp1, planet_name[i]);
			strcat(lptemp1, " Semi-sextile ");
			strcat(lptemp1, planet_name[j]);
			strcat(lptemp1, "\r\n");
			temp=1;
			if ((k==30) || (k==330)) temp=2;
			DeleteObject(SelectObject(mapdc, CreatePen(PS_SOLID,temp,0xFF8000)));
			aspects[i]++;
			aspects[j]++;
		};
		// Opposition 180
		if (k>=173 && k<=187)
		{
			strcat(lptemp1, planet_name[i]);
			strcat(lptemp1, " Opposition ");
			strcat(lptemp1, planet_name[j]);
			strcat(lptemp1, "\r\n");
			temp=1;
			if (k==180.0) temp=2;
			DeleteObject(SelectObject(mapdc, CreatePen(PS_SOLID,temp,0xC00000)));
			aspects[i]++;
			aspects[j]++;
		};
		// Quincunx 150 - 210
		if ((k>=143 && k<=157) || (k>=203 && k<=217))
		{
			strcat(lptemp1, planet_name[i]);
			strcat(lptemp1, " Quincunx ");
			strcat(lptemp1, planet_name[j]);
			strcat(lptemp1, "\r\n");
			temp=1;
			if ((k==150) || (k==210)) temp=2;
			DeleteObject(SelectObject(mapdc, CreatePen(PS_SOLID,temp,0x008080)));
			aspects[i]++;
			aspects[j]++;
		};
		if (temp)
		{
			MoveToEx(mapdc, cos(Convert(planets[i]))*100+250,sin(Convert(planets[i]))*100+250,NULL);
			LineTo(mapdc, cos(Convert(planets[j]))*100+250,sin(Convert(planets[j]))*100+250);
		};
	};

	// Shows aspect totals
	temp = 0;
	strcat(lptemp1, "\r\nAspects totals\r\n\r\n");
	for (i=SE_SUN; i<=SE_MEAN_NODE+2; i++)
	{
		strcat(lptemp1, planet_name[i]);
		strcat(lptemp1, ":\t");
		strcat(lptemp1, itoa(aspects[i],lptemp2, 10));
		strcat(lptemp1, "\r\n");
		temp+=aspects[i];
	};
	strcat(lptemp1, "Total:\t");
	strcat(lptemp1, itoa(temp / 2, lptemp2, 10));
	strcat(lptemp1, " / 78");

	// Finishes the graphic
	DeleteObject(SelectObject(mapdc, CreatePen(PS_SOLID,1,0x000000)));
	lb.lbColor = 0x000000;
	lb.lbStyle = BS_HOLLOW;
	DeleteObject(SelectObject(mapdc, CreateBrushIndirect(&lb)));
	Ellipse(mapdc, 150,150,350,350);
	BitBlt(hdc, 0,0,500,500,mapdc,0,0,SRCCOPY);

	SetWindowText(memo, lptemp1);

	swe_close();
};

LRESULT CALLBACK WindowProc(HWND hwnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	if (uMsg==WM_DESTROY)
	{
		PostQuitMessage(0);
		return 0;
	};
	if (uMsg==WM_COMMAND)
	{
		if (LOWORD(wParam)==IDOK) Calculate();

		if (LOWORD(wParam)==IDNOW)
		{
			//Hora
			SYSTEMTIME st;
			GetLocalTime(&st);
			SetWindowText(edtd, itoa(st.wDay, lptemp1, 10));
			SetWindowText(edtm, itoa(st.wMonth, lptemp1, 10));
			SetWindowText(edta, itoa(st.wYear, lptemp1, 10));
			SetWindowText(edth, itoa(st.wHour, lptemp1, 10));
			SetWindowText(edtmin, itoa(st.wMinute, lptemp1, 10));
			SetWindowText(edtseg, itoa(st.wSecond, lptemp1, 10));
			Calculate();
		};
		if (LOWORD(wParam)==IDABOUT)
			MessageBox(window, "BRELS AstroSource v0.2\r\nby Breno Sarmento\r\n\r\nThis software is freeware. Send any comments to breno_sarmento@hotmail.com\r\n\r\nThe Swiss Ephemeris file (Swedll32.dll) is freeware only for freeware software.\r\n\r\nNote: provisorily using the \"Astro\" font. I'm working on an Unicode font to avoid copyright issues.\r\n\r\nCreated March 24th, 2003. Released November 24th, 2003", "About", MB_OK | MB_ICONINFORMATION);

		return 0;
	}
	if ((uMsg==WM_PAINT) && (hwnd==map))
	{
		PAINTSTRUCT lpps;
		BeginPaint(map, &lpps);
		BitBlt(hdc,0,0,500,500,mapdc,0,0,SRCCOPY);
		EndPaint(map, &lpps);
		return 0;
	};
	return DefWindowProc(hwnd, uMsg, wParam, lParam);
};

int WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
{
	// Initialization
	BRELS = (HICON) LoadImage(hInstance, "logo.ico", IMAGE_ICON, 16,16,LR_LOADFROMFILE | LR_DEFAULTSIZE);

	// Main window
	wc.style = CS_OWNDC | CS_DBLCLKS;
	wc.lpfnWndProc = WindowProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = BRELS;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH) (COLOR_BTNFACE+1);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = "WINDOW";
	RegisterClass(&wc);

	// Map window
	wc.hbrBackground =(HBRUSH) (COLOR_WINDOW+1);
	wc.lpszClassName = "MAPA";
	RegisterClass(&wc);

	window=CreateWindow("WINDOW", "BRELS AstroSource v0.2", WS_CAPTION | WS_MINIMIZEBOX | WS_SYSMENU, 0,0,752 + 2*GetSystemMetrics(SM_CXFIXEDFRAME),532 + GetSystemMetrics(SM_CYCAPTION) + 2*GetSystemMetrics(SM_CYFIXEDFRAME),NULL, NULL, hInstance, NULL);
	CreateWindow("STATIC", "Date (d/m/y)", WS_CHILD | WS_VISIBLE | WS_VISIBLE, 4,5,132,20,window, NULL, hInstance, NULL);
	CreateWindow("STATIC", "Time (24h)", WS_CHILD | WS_VISIBLE | WS_VISIBLE, 176,5,132,20,window, NULL, hInstance, NULL);
	CreateWindow("STATIC", "GMT", WS_CHILD | WS_VISIBLE | WS_VISIBLE, 328,5,132,20,window, NULL, hInstance, NULL);
	CreateWindow("STATIC", "Lat.", WS_CHILD | WS_VISIBLE | WS_VISIBLE, 400,5,132,20,window, NULL, hInstance, NULL);
	CreateWindow("STATIC", "Lon.", WS_CHILD | WS_VISIBLE | WS_VISIBLE, 496,5,132,20,window, NULL, hInstance, NULL);
	CreateWindow("STATIC", "(S/W -)", WS_CHILD | WS_VISIBLE | WS_VISIBLE, 584,5,132,20,window, NULL, hInstance, NULL);
	edtd = CreateWindow("EDIT", "31", WS_TABSTOP | WS_VISIBLE | WS_CHILD | WS_BORDER, 84,4,20,20,window, NULL, hInstance, NULL);
	edtm = CreateWindow("EDIT", "07", WS_TABSTOP | WS_VISIBLE | WS_CHILD | WS_BORDER, 108,4,20, 20,window, NULL, hInstance, NULL);
	edta = CreateWindow("EDIT", "1982", WS_TABSTOP | WS_VISIBLE | WS_CHILD | WS_BORDER, 132,4,40,20,window, NULL, hInstance, NULL);
	edth = CreateWindow("EDIT", "22", WS_TABSTOP | WS_VISIBLE | WS_CHILD | WS_BORDER, 248,4,20,20,window, NULL, hInstance, NULL);
	edtmin = CreateWindow("EDIT", "30", WS_TABSTOP | WS_VISIBLE | WS_CHILD | WS_BORDER, 272,4,20,20,window, NULL, hInstance, NULL);
	edtseg = CreateWindow("EDIT", "00", WS_TABSTOP | WS_VISIBLE | WS_CHILD | WS_BORDER, 296,4,20,20,window, NULL, hInstance, NULL);
	edtgmt = CreateWindow("EDIT", "-3", WS_TABSTOP | WS_VISIBLE | WS_CHILD | WS_BORDER, 360,4,20,20,window, NULL, hInstance, NULL);
	edtlatg = CreateWindow("EDIT", "-1", WS_TABSTOP | WS_VISIBLE | WS_CHILD | WS_BORDER, 428,4,24,20,window, NULL, hInstance, NULL);
	edtlatm = CreateWindow("EDIT", "27", WS_TABSTOP | WS_VISIBLE | WS_CHILD | WS_BORDER, 456,4,24,20,window, NULL, hInstance, NULL);
	edtlong = CreateWindow("EDIT", "-48", WS_TABSTOP | WS_VISIBLE | WS_CHILD | WS_BORDER, 528,4,24,20,window, NULL, hInstance, NULL);
	edtlonm = CreateWindow("EDIT", "30", WS_TABSTOP | WS_VISIBLE | WS_CHILD | WS_BORDER, 556,4,24,20,window, NULL, hInstance, NULL);
	map = CreateWindow("MAPA", "", WS_VISIBLE | WS_CHILD | WS_BORDER, 4,28,501,501,window, NULL, hInstance, NULL);
	btcalc = CreateWindow("BUTTON", "&Go!", WS_VISIBLE | WS_CHILD | BS_DEFPUSHBUTTON | WS_TABSTOP, 636,4, 36, 20, window, (HMENU) IDOK, hInstance, NULL);
	btnow = CreateWindow("BUTTON", "&Now!", WS_VISIBLE | WS_CHILD | WS_TABSTOP, 678,4, 36, 20, window, (HMENU) IDNOW, hInstance, NULL);
	btabout = CreateWindow("BUTTON", "?", WS_VISIBLE | WS_CHILD | WS_TABSTOP, 718,4, 28, 20, window, (HMENU) IDABOUT, hInstance, NULL);
	memo = CreateWindow("EDIT", "", WS_VISIBLE | WS_CHILD | ES_MULTILINE | WS_BORDER | WS_TABSTOP | WS_VSCROLL, 508,28,240,501,window, NULL, hInstance, NULL);

	hdc = GetDC(map);
	mapdc = CreateCompatibleDC(hdc);
	SelectObject(mapdc, CreateCompatibleBitmap(hdc, 500,500));
	SelectObject(mapdc, CreateFont(28, 0, 0, 0, 900, FALSE, FALSE, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH, "astro"));

	Calculate();

	ShowWindow(window, nCmdShow);
	
	while( GetMessage( &msg, NULL, 0, 0 ) > 0) 
	{ 
		if (IsDialogMessage(GetActiveWindow(),&msg)==0)
		{	
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		};
	};

	ReleaseDC(map, hdc);	
	DeleteDC(mapdc);
	DestroyIcon(BRELS);

	return msg.wParam;
};
